(* libguestfs generated file
 * WARNING: THIS FILE IS GENERATED FROM THE FOLLOWING FILES:
 *          generator/daemon.ml
 *          and from the code in the generator/ subdirectory.
 * ANY CHANGES YOU MAKE TO THIS FILE WILL BE LOST.
 *
 * Copyright (C) 2009-2019 Red Hat Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 *)

val acl_available : unit -> bool
val blkdiscard_available : unit -> bool
val blkdiscardzeroes_available : unit -> bool
val btrfs_available : unit -> bool
val extlinux_available : unit -> bool
val f2fs_available : unit -> bool
val fstrim_available : unit -> bool
val gdisk_available : unit -> bool
val grub_available : unit -> bool
val hivex_available : unit -> bool
val inotify_available : unit -> bool
val journal_available : unit -> bool
val ldm_available : unit -> bool
val libtsk_available : unit -> bool
val libyara_available : unit -> bool
val linuxcaps_available : unit -> bool
val linuxfsuuid_available : unit -> bool
val linuxmodules_available : unit -> bool
val linuxxattrs_available : unit -> bool
val luks_available : unit -> bool
val lvm2_available : unit -> bool
val mdadm_available : unit -> bool
val mknod_available : unit -> bool
val ntfs3g_available : unit -> bool
val ntfsprogs_available : unit -> bool
val rsync_available : unit -> bool
val scrub_available : unit -> bool
val selinux_available : unit -> bool
val selinuxrelabel_available : unit -> bool
val sleuthkit_available : unit -> bool
val squashfs_available : unit -> bool
val syslinux_available : unit -> bool
val wipefs_available : unit -> bool
val xfs_available : unit -> bool
val xz_available : unit -> bool
val zerofree_available : unit -> bool
